@@ -505,6 +505,12 @@ int ccid_open_hack_post(unsigned int reader_index)
 			ccid_descriptor->dwFeatures |= CCID_CLASS_EXTENDED_APDU;
 			break;
 
+		case KOBIL_TRIBANK:
+			/* Firmware does NOT supported extended APDU */
+			ccid_descriptor->dwFeatures &= ~CCID_CLASS_EXCHANGE_MASK;
+			ccid_descriptor->dwFeatures |= CCID_CLASS_SHORT_APDU;
+			break;
+
 #if 0
 		/* SCM SCR331-DI contactless */
 		case SCR331DI:
@@ -564,6 +570,12 @@ int ccid_open_hack_post(unsigned int reader_index)
 			/* restore default timeout (modified in ccid_open_hack_pre()) */
 			ccid_descriptor->readTimeout = DEFAULT_COM_READ_TIMEOUT;
 			break;
+
+		case BIT4ID_MINILECTOR:
+			/* The firmware 1.11 advertises pinpad but actually doesn't
+			 * have one */
+			ccid_descriptor->bPINSupport = 0;
+			break;
 	}
 
 	/* Gemalto readers may report additional information */
