@@ -268,7 +268,7 @@ again:
 		if (voltage)
 		{
 #ifndef NO_LOG
-			const char *voltage_code[] = { "auto", "5V", "3V", "1.8V" };
+			const char *voltage_code[] = { "1.8V", "5V", "3V", "1.8V" };
 #endif
 
 			DEBUG_INFO3("Power up with %s failed. Try with %s.",
@@ -390,6 +390,18 @@ RESPONSECODE SecurePINVerify(unsigned int reader_index,
 
 	}
 
+	if ((VENDOR_GEMALTO == GET_VENDOR(ccid_descriptor->readerID))
+		&& (ccid_descriptor->gemalto_firmware_features))
+	{
+		int bEntryValidationCondition = ccid_descriptor->gemalto_firmware_features->bEntryValidationCondition;
+		if (TxBuffer[7] & ~bEntryValidationCondition)
+		{
+			DEBUG_INFO2("Correct bEntryValidationCondition (was 0x%02X)",
+				TxBuffer[7]);
+			TxBuffer[7] &= bEntryValidationCondition;
+		}
+	}
+
 	if ((DELLSCRK == ccid_descriptor->readerID)
 		|| (DELLSK == ccid_descriptor->readerID))
 	{
@@ -747,6 +759,18 @@ RESPONSECODE SecurePINModify(unsigned int reader_index,
 		}
 	}
 
+	if ((VENDOR_GEMALTO == GET_VENDOR(ccid_descriptor->readerID))
+		&& (ccid_descriptor->gemalto_firmware_features))
+	{
+		int bEntryValidationCondition = ccid_descriptor->gemalto_firmware_features->bEntryValidationCondition;
+		if (TxBuffer[10] & ~bEntryValidationCondition)
+		{
+			DEBUG_INFO2("Correct bEntryValidationCondition (was 0x%02X)",
+				TxBuffer[10]);
+			TxBuffer[10] &= bEntryValidationCondition;
+		}
+	}
+
 	gemalto_modify_pin_bug = has_gemalto_modify_pin_bug(ccid_descriptor);
 	if (gemalto_modify_pin_bug)
 	{
@@ -1864,7 +1888,7 @@ static RESPONSECODE T0ProcACK(unsigned int reader_index,
 
 	DEBUG_COMM2("Enter, is_rcv = %d", is_rcv);
 
-	if (proc_len > 0x20000)
+	if (proc_len > 0x200)
 		return IFD_COMMUNICATION_ERROR;
 
 	if (is_rcv == 1)
@@ -1912,29 +1936,29 @@ static RESPONSECODE T0ProcACK(unsigned int reader_index,
 
 #ifdef O2MICRO_OZ776_PATCH
 		if((0 != remain_len) && (0 == (remain_len + 10) % 64))
-        {
-			/* special hack to avoid a command of size modulo 64
-			 * we send two commands instead */
-            ret_len = 1;
-            return_value = CCID_Transmit(reader_index, 0, *snd_buf, ret_len, 0);
-            if (return_value != IFD_SUCCESS)
-                return return_value;
-            return_value = CCID_Receive(reader_index, &ret_len, tmp_buf, NULL);
-            if (return_value != IFD_SUCCESS)
-                return return_value;
+		{
+		    /* special hack to avoid a command of size modulo 64
+		     * we send two commands instead */
+		    ret_len = 1;
+		    return_value = CCID_Transmit(reader_index, 0, *snd_buf, ret_len, 0);
+		    if (return_value != IFD_SUCCESS)
+				return return_value;
+		    return_value = CCID_Receive(reader_index, &ret_len, tmp_buf, NULL);
+		    if (return_value != IFD_SUCCESS)
+				return return_value;
 
-            ret_len = remain_len - 1;
-            return_value = CCID_Transmit(reader_index, 0, *snd_buf, ret_len, 0);
-            if (return_value != IFD_SUCCESS)
-                return return_value;
-            return_value = CCID_Receive(reader_index, &ret_len, &tmp_buf[1],
-				NULL);
-            if (return_value != IFD_SUCCESS)
-                return return_value;
+		    ret_len = remain_len - 1;
+		    return_value = CCID_Transmit(reader_index, 0, *snd_buf, ret_len, 0);
+		    if (return_value != IFD_SUCCESS)
+				return return_value;
+		    return_value = CCID_Receive(reader_index, &ret_len, &tmp_buf[1],
+			    NULL);
+		    if (return_value != IFD_SUCCESS)
+				return return_value;
 
-            ret_len += 1;
-        }
-        else
+		    ret_len += 1;
+		}
+		else
 #endif
 		{
 			ret_len = remain_len;
