@@ -476,9 +476,9 @@ EXTERNAL RESPONSECODE IFDHGetCapabilities(DWORD Lun, DWORD Tag,
 						|| (HID_OMNIKEY_5422 == readerID))
 						*Value = 2;
 
-					/* 3 CCID interfaces */
+					/* 4 CCID interfaces */
 					if (FEITIANR502DUAL == readerID)
-						*Value = 3;
+						*Value = 4;
 				}
 #endif
 				DEBUG_INFO2("Reader supports %d slot(s)", *Value);
@@ -723,14 +723,6 @@ EXTERNAL RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 	ccid_slot = get_ccid_slot(reader_index);
 	ccid_desc = get_ccid_descriptor(reader_index);
 
-	/* Do not send CCID command SetParameters or PPS to the CCID
-	 * The CCID will do this himself */
-	if (ccid_desc->dwFeatures & CCID_CLASS_AUTO_PPS_PROP)
-	{
-		DEBUG_COMM2("Timeout: %d ms", ccid_desc->readTimeout);
-		goto end;
-	}
-
 	/* check the protocol is supported by the reader */
 	if (!(Protocol & ccid_desc->dwProtocols))
 	{
@@ -796,6 +788,14 @@ EXTERNAL RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 			}
 	}
 
+    /* Do not send CCID command SetParameters or PPS to the CCID
+	 * The CCID will do this himself */
+	if (ccid_desc->dwFeatures & CCID_CLASS_AUTO_PPS_PROP)
+	{
+		DEBUG_COMM2("Timeout: %d ms", ccid_desc->readTimeout);
+		goto end;
+	}
+
 	/* PTS1? */
 	if (Flags & IFD_NEGOTIATE_PTS1)
 	{
@@ -1092,8 +1092,10 @@ EXTERNAL RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 			return ret;
 	}
 
+end:
 	/* set IFSC & IFSD in T=1 */
-	if (SCARD_PROTOCOL_T1 == Protocol)
+	if ((SCARD_PROTOCOL_T1 == Protocol)
+		&& (CCID_CLASS_TPDU == (ccid_desc->dwFeatures & CCID_CLASS_EXCHANGE_MASK)))
 	{
 		t1_state_t *t1 = &(ccid_slot -> t1);
 		int i, ifsc;
@@ -1117,7 +1119,6 @@ EXTERNAL RESPONSECODE IFDHSetProtocolParameters(DWORD Lun, DWORD Protocol,
 		DEBUG_COMM3("T=1: IFSC=%d, IFSD=%d", t1->ifsc, t1->ifsd);
 	}
 
-end:
 	/* store used protocol for use by the secure commands (verify/change PIN) */
 	ccid_desc->cardProtocol = Protocol;
 
