@@ -74,14 +74,15 @@ int main(int argc, char *argv[])
 	r = libusb_init(NULL);
 	if (r < 0)
 	{
-		(void)printf("libusb_init() failed\n");
+		(void)printf("libusb_init() failed: %s\n", libusb_error_name(r));
         return r;
 	}
 
 	cnt = libusb_get_device_list(NULL, &devs);
     if (cnt < 0)
 	{
-		(void)printf("libusb_get_device_list() failed\n");
+		(void)printf("libusb_get_device_list() failed: %s\n",
+			libusb_error_name(r));
         return (int)cnt;
 	}
 
@@ -116,7 +117,8 @@ int main(int argc, char *argv[])
         if (r < 0)
 		{
             (void)fprintf(stderr,
-				BRIGHT_RED "failed to get device descriptor" NORMAL);
+				BRIGHT_RED "failed to get device descriptor: %s" NORMAL,
+				libusb_error_name(r));
             return 1;
         }
 
@@ -130,7 +132,8 @@ int main(int argc, char *argv[])
 			buffer, sizeof(buffer));
 		if (r < 0)
 		{
-			(void)fprintf(stderr, "  Can't get iManufacturer string\n");
+			(void)fprintf(stderr, "  Can't get iManufacturer string: %s\n",
+				libusb_error_name(r));
 			if (getuid())
 			{
 				(void)fprintf(stderr,
@@ -146,7 +149,8 @@ int main(int argc, char *argv[])
 		r = libusb_get_string_descriptor_ascii(handle, desc.iProduct,
 			buffer, sizeof(buffer));
 		if (r < 0)
-			(void)fprintf(stderr, "  Can't get iProduct string\n");
+			(void)fprintf(stderr, "  Can't get iProduct string: %s\n",
+				libusb_error_name(r));
 		else
 			(void)fprintf(stderr, "  iProduct: " BLUE "%s\n" NORMAL, buffer);
 
@@ -155,7 +159,8 @@ again:
 		r = libusb_get_active_config_descriptor(dev, &config_desc);
 		if (r < 0)
 		{
-			(void)fprintf(stderr, "  Can't get config descriptor: %d\n", r);
+			(void)fprintf(stderr, "  Can't get config descriptor: %s\n",
+				libusb_error_name(r));
 			(void)libusb_close(handle);
 			continue;
 		}
@@ -262,7 +267,8 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 		buffer, sizeof(buffer));
 	if (r < 0)
 	{
-		(void)printf("  Can't get iManufacturer string\n");
+		(void)printf("  Can't get iManufacturer string: %s\n",
+			libusb_error_name(r));
 		if (getuid())
 		{
 			(void)fprintf(stderr,
@@ -277,7 +283,8 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 	r =  libusb_get_string_descriptor_ascii(handle, desc.iProduct,
 		buffer, sizeof(buffer));
 	if (r < 0)
-		(void)printf("  Can't get iProduct string\n");
+		(void)printf("  Can't get iProduct string: %s\n",
+			libusb_error_name(r));
 	else
 		(void)printf("  iProduct: %s\n", buffer);
 
@@ -350,7 +357,8 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 	r = libusb_get_string_descriptor_ascii(handle, usb_interface_descriptor->iInterface,
 		buffer, sizeof(buffer));
 	if (r < 0)
-		(void)printf(" Can't get iInterface string\n");
+		(void)printf(" Can't get iInterface string: %s\n",
+			libusb_error_name(r));
 	else
 		(void)printf(" iInterface: %s\n", buffer);
 
@@ -404,11 +412,16 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 
 	(void)printf("  dwDefaultClock: %.3f MHz\n", dw2i(device_descriptor, 10)/1000.0);
 	(void)printf("  dwMaximumClock: %.3f MHz\n", dw2i(device_descriptor, 14)/1000.0);
-	(void)printf("  bNumClockSupported: %d%s\n", device_descriptor[18],
-		device_descriptor[18] ? "" : " (will use whatever is returned)");
+	int bNumClockSupported = device_descriptor[18];
+	(void)printf("  bNumClockSupported: %d%s\n", bNumClockSupported,
+		bNumClockSupported ? "" : " (will use whatever is returned)");
 	{
 		int n;
 
+		if (0 == bNumClockSupported)
+			/* read up to the buffer size */
+			bNumClockSupported = sizeof(buffer) / sizeof(int);
+
 		/* See CCID 5.3.2 page 24 */
 		n = libusb_control_transfer(handle,
 			0xA1, /* request type */
@@ -416,7 +429,7 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 			0x00, /* value */
 			usb_interface_descriptor->bInterfaceNumber, /* interface */
 			buffer,
-			sizeof(buffer),
+			bNumClockSupported * sizeof(int),
 			2 * 1000);
 
 		/* we got an error? */
@@ -438,15 +451,15 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 				int i;
 
 				/* we do not get the expected number of data rates */
-				if ((n != device_descriptor[18]*4) && device_descriptor[18])
+				if ((n != bNumClockSupported*4) && bNumClockSupported)
 				{
 					(void)printf("   Got %d clock frequencies but was expecting %d\n",
-						n/4, device_descriptor[18]);
+						n/4, bNumClockSupported);
 
 					/* we got more data than expected */
 #ifndef DISPLAY_EXTRA_VALUES
-					if (n > device_descriptor[18]*4)
-						n = device_descriptor[18]*4;
+					if (n > bNumClockSupported*4)
+						n = bNumClockSupported*4;
 #endif
 				}
 
@@ -456,11 +469,16 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 	}
 	(void)printf("  dwDataRate: %d bps\n", dw2i(device_descriptor, 19));
 	(void)printf("  dwMaxDataRate: %d bps\n", dw2i(device_descriptor, 23));
-	(void)printf("  bNumDataRatesSupported: %d%s\n", device_descriptor[27],
-		device_descriptor[27] ? "" : " (will use whatever is returned)");
+	int bNumDataRatesSupported = device_descriptor[27];
+	(void)printf("  bNumDataRatesSupported: %d%s\n", bNumDataRatesSupported,
+		bNumDataRatesSupported ? "" : " (will use whatever is returned)");
 	{
 		int n;
 
+		if (0 == bNumDataRatesSupported)
+			/* read up to the buffer size */
+			bNumDataRatesSupported = sizeof(buffer) / sizeof(int);
+
 		/* See CCID 5.3.3 page 24 */
 		n = libusb_control_transfer(handle,
 			0xA1, /* request type */
@@ -468,7 +486,7 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 			0x00, /* value */
 			usb_interface_descriptor->bInterfaceNumber, /* interface */
 			buffer,
-			sizeof(buffer),
+			bNumDataRatesSupported * sizeof(int),
 			2 * 1000);
 
 		/* we got an error? */
@@ -483,15 +501,15 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 				int i;
 
 				/* we do not get the expected number of data rates */
-				if ((n != device_descriptor[27]*4) && device_descriptor[27])
+				if ((n != bNumDataRatesSupported*4) && bNumDataRatesSupported)
 				{
 					(void)printf("   Got %d data rates but was expecting %d\n", n/4,
-						device_descriptor[27]);
+						bNumDataRatesSupported);
 
 					/* we got more data than expected */
 #ifndef DISPLAY_EXTRA_VALUES
-					if (n > device_descriptor[27]*4)
-						n = device_descriptor[27]*4;
+					if (n > bNumDataRatesSupported*4)
+						n = bNumDataRatesSupported*4;
 #endif
 				}
 
@@ -544,7 +562,7 @@ static int ccid_parse_interface_descriptor(libusb_device_handle *handle,
 	if (device_descriptor[41] & 0x04)
 		(void)printf("   ..04.. Automatic IFSD exchange as first exchange (T=1)\n");
 	if (device_descriptor[41] & 0x08)
-		(void)printf("   ..08.. Unknown (ICCD?)\n");
+		(void)printf("   ..08.. ICCD token\n");
 	switch (device_descriptor[42] & 0x07)
 	{
 		case 0x00:
